@@ -9,7 +9,7 @@ unset LC_CTYPE LANG
 FINDBIN=$(cd -- "$(dirname "$0")" && pwd)
 BASEDIR="$(cd "$FINDBIN/.." && pwd -P)"
 
-SANDBOX="${BASEDIR}/sandbox"
+SANDBOX=${SANDBOX:-${BASEDIR}/sandbox}
 # shellcheck disable=SC2034
 PERLLIB=$SANDBOX$(perl -V:sitelib | cut -d"'" -f2)
 
