@@ -7,7 +7,7 @@ import java.lang.management.ThreadMXBean;
 import org.munin.plugin.jmx.AbstractAnnotationGraphsProvider.Graph;
 
 
-@Graph(title = "CurrentThreadCpuTime", vlabel = "ns", info = "Returns the total CPU time for the current thread in nanoseconds. The returned value is of nanoseconds precison but not necessarily nanoseconds accuracy. If the implementation distinguishes between user mode time and system mode time, the returned CPU time is the amount of time that the current thread has executed in user mode or system mode.")
+@Graph(title = "CurrentThreadCpuTime", vlabel = "ns", info = "Returns the total CPU time for the current thread in nanoseconds. The returned value is of nanoseconds precision but not necessarily nanoseconds accuracy. If the implementation distinguishes between user mode time and system mode time, the returned CPU time is the amount of time that the current thread has executed in user mode or system mode.")
 public class CurrentThreadCpuTime extends AbstractAnnotationGraphsProvider {
 
 	public CurrentThreadCpuTime(Config config) {
